/// <reference types="node" />
import { Hook, Loader, Mapper, Matcher, StrictLoader, StrictLoaderWithOption } from '../../types/TemplateConfig';
import { AwaitEventEmitter, Logger, Tree } from '../../types/core';
import VariablesImpl from './Variables';
export declare type Asset = {
    value: Buffer | string | any;
    loaders?: Array<StrictLoader>;
};
export default class Compiler extends AwaitEventEmitter {
    logger: Logger;
    root: string;
    hookCwd: string;
    includes: Matcher;
    excludes: Matcher;
    removeHook(hookName: string, hook?: Function): this;
    addHook(hookName: string, hook: Hook, options?: {
        type?: 'on' | 'once';
        silent?: boolean;
    }): Function;
    constructor({ loaders, mappers, hookCwd, root, includes, excludes }?: {
        excludes?: Matcher;
        includes?: Matcher;
        loaders?: any;
        mappers?: any;
        root?: string;
        hookCwd?: string;
    });
    addLoader(loaderId: string, loader: Loader): Compiler;
    removeLoader(loaderId: string): Compiler;
    assets: {
        [path: string]: Asset;
    };
    loaders: {
        [loaderId: string]: Array<StrictLoader | StrictLoaderWithOption>;
    };
    mappers: Array<Mapper>;
    variables: VariablesImpl;
    private _matchedLoaders;
    transform(input: any, loaders: Loader, path: string, highOrderOptions: object): Promise<any>;
    run(): Promise<Tree>;
}
