"use strict";
/**
 * @file plopHandlebar
 * @author Cuttle Cong
 * @date 2018/3/25
 * @description
 */
Object.defineProperty(exports, "__esModule", { value: true });
const Handlebars = require("handlebars");
const changeCase = require("change-case");
const detectLegacyLodash_1 = require("./detectLegacyLodash");
const lineColPath = require("line-column-path");
const link = require("terminal-link");
const chalk_1 = require("chalk");
// Creates an isolated Handlebars environment
const handlebars = Handlebars.create();
const helpers = Object.assign({}, changeCase, { camelCase: changeCase.camel, snakeCase: changeCase.snake, dotCase: changeCase.dot, pathCase: changeCase.path, lowerCase: changeCase.lower, upperCase: changeCase.upper, sentenceCase: changeCase.sentence, constantCase: changeCase.constant, titleCase: changeCase.title, dashCase: changeCase.param, kabobCase: changeCase.param, kebabCase: changeCase.param, properCase: changeCase.pascal, pascalCase: changeCase.pascal });
// Register helpers
Object.keys(helpers).forEach(h => handlebars.registerHelper(h, helpers[h]));
// https://github.com/helpers/handlebars-helpers
require('handlebars-helpers')({
    handlebars: handlebars
});
function hbsLoader(content, variables) {
    // Object.keys(partials).forEach(p => handlebars.registerPartial(p, partials[p]))
    const { options, path, compiler } = this;
    const error = compiler && typeof compiler.logger.error === 'function'
        ? compiler.logger.error
        : console.error;
    if (typeof options.process === 'function') {
        options.process(handlebars);
    }
    const posList = detectLegacyLodash_1.default(content, variables);
    if (posList.length) {
        posList.forEach(pos => {
            error(chalk_1.default.whiteBright.bgRed(lineColPath.stringify(Object.assign({ file: path }, pos))), `Detected using the deprecated Lodash loader, please ${link('replace it', 'https://bit.ly/2CYM8lC')} by handlebar.`);
        });
    }
    return handlebars.compile(content)(variables);
}
module.exports = hbsLoader;
